#include "BSpline.h"
#include <stdlib.h>
#include <vector>

/*********************************************************************

 Simple b-spline curve algorithm

 Copyright 1994 by Keith Vertanen (vertankd@cda.mrs.umn.edu)

 Released to the public domain (your mileage may vary)

**********************************************************************/

/**
 * Constructor
 */
BSpline::BSpline(void) {
	controlPoints.clear();

	nofControlPoints = -1;
}

/**
 * Destructor. Clean up evrything
 */
BSpline::~BSpline(void) {
	for(BSPointArrayIterator iter = controlPoints.begin(); iter != controlPoints.end(); iter++) {
		BSplinePoint *point = *iter;

		// delete given point
		delete *iter;
	}
}

/**
 * Add a 3d point to the list of control points
 *
 * @args points[3]  array of x,y,z points
 */
void BSpline::addControlPoint(double points[3]) {
	BSplinePoint *point = new BSplinePoint();

	point->x = points[0];
	point->y = points[1];
	point->z = points[2];

	nofControlPoints++;
	controlPoints.push_back(point);
}

/**
 * Add a 3d point to the list of control points
 *
 * @args x   x coordinate
 * @args y   y coordinate
 * @args z   z coordinate
 */
void BSpline::addControlPoint(double x, double y, double z) {
	BSplinePoint *point = new BSplinePoint();

	point->x = x;
	point->y = y;
	point->z = z;

	nofControlPoints++;
	controlPoints.push_back(point);
}



/**
 * 

Parameters:
  n          - the number of control points minus 1
  t          - the degree of the polynomial plus 1
  control    - control point array made up of point stucture
  output     - array in which the calculate spline points are to be put
  num_output - how many points on the spline are to be calculated

Pre-conditions:
  n+2>t  (no curve results if n+2<=t)
  control array contains the number of points specified by n
  output array is the proper size to hold num_output point structures

**********************************************************************/
void BSpline::bspline(int degree, double output[61][3], int num_output) {
	int n = nofControlPoints;

	if (n+2 > degree) {
		/* n+1 > degree must hold, otherwise no curve can be computed */
		//assert(0);
	}

  int *u;
  double increment,interval;
  double calcxyz[3];
  int output_index;

  u = new int[n+degree+1];
  compute_intervals(u, degree);

  increment=(double) (n-degree+2)/(num_output-1);  // how much parameter goes up each time
  interval=0;

  for (output_index=0; output_index<num_output-1; output_index++)
  {
    computePoint(u, degree, interval, calcxyz);
    output[output_index][0] = calcxyz[0];
    output[output_index][1] = calcxyz[1];
    output[output_index][2] = calcxyz[2];
    interval=interval+increment;  // increment our parameter

	//printf("interval: %f\n", interval);
  }
  output[num_output-1][0] = controlPoints[n]->x;   // put in the last point
  output[num_output-1][1] = controlPoints[n]->y;
  output[num_output-1][2] = controlPoints[n]->z;

  delete u;
}




/**
 *
 */
void BSpline::compute_intervals(int *u, int degree) {   // figure out the knots
	int j;

	int n = nofControlPoints;

	for(j=0; j<=n+degree; j++) {
		if(j < degree) {
			u[j] = 0;
		}
		else {
			if((degree <= j) && (j <= n)) {
				u[j] = j - degree +1;
			}
			else {
				if(j > n) {
					u[j] = n - degree +2;  // if n-degree=-2 then we're screwed, everything goes to 0
				}
			}
		}
	}
}
/*
void compute_intervals(int *u, int n, int t)   // figure out the knots
{
  int j;

  for (j=0; j<=n+t; j++)
  {
    if (j<t)
      u[j]=0;
    else
    if ((t<=j) && (j<=n))
      u[j]=j-t+1;
    else
    if (j>n)
      u[j]=n-t+2;  // if n-t=-2 then we're screwed, everything goes to 0
  }
}
*/


/**
 *
 */
void BSpline::computePoint(int *u, int degree, double interval, double *output) {
	int n = nofControlPoints;

	// initialize the variables that will hold our outputted point
	output[0] = 0.0;
	output[1] = 0.0;
	output[2] = 0.0;


	

	int i=0;
	for(BSPointArrayIterator iter = controlPoints.begin(); iter != controlPoints.end(); iter++) {
		BSplinePoint *point = *iter;

		// get blend coefficient
		double temp = blend(i, degree, u, interval);

		// sum up controlPonints with their blend coefficient
		output[0] = output[0] + point->x * temp;
		output[1] = output[1] + point->y * temp;
		output[2] = output[2] + point->z * temp;

		// next point
		i++;
	}
}
/*
void compute_point(int *u, int n, int t, double v, point *control,
			point *output)
{
  int k;
  double temp;

  // initialize the variables that will hold our outputted point
  output->x=0;
  output->y=0;
  output->z=0;

  for (k=0; k<=n; k++)
  {
    temp = blend(k,t,u,v);  // same blend is used for each dimension coordinate
    output->x = output->x + (control[k]).x * temp;
    output->y = output->y + (control[k]).y * temp;
    output->z = output->z + (control[k]).z * temp;
  }
}
*/

/**
 * Bend
 */
double BSpline::blend(int pointIndex, int degree, int *u, double interval) {
	double weight = 0.0;

	if (degree == 1) {
		if (u[pointIndex] <= interval && interval < u[pointIndex+1]) {
			weight = 1.0;
		}
		else {
			weight = 0.0;
		}
	}
	else {
		if ((u[pointIndex+degree-1] == u[pointIndex]) && (u[pointIndex+degree] == u[pointIndex+1])) {
			weight = 0.0;
		}
		else {
			if (u[pointIndex+degree-1] == u[pointIndex]) {
				weight = (u[pointIndex+degree] - interval) / (u[pointIndex+degree] - u[pointIndex+1]) * blend(pointIndex+1, degree-1, u, interval);
			}
			else {
				if (u[pointIndex+degree]==u[pointIndex+1]) {
					weight = (interval - u[pointIndex]) / (u[pointIndex+degree-1] - u[pointIndex]) * blend(pointIndex, degree-1, u, interval);
				}
				else {
					weight = (interval - u[pointIndex]) / (u[pointIndex+degree-1] - u[pointIndex]) * blend(pointIndex, degree-1, u, interval) +
					         (u[pointIndex+degree] - interval) / (u[pointIndex+degree] - u[pointIndex+1]) * blend(pointIndex+1, degree-1, u, interval);
				}
			}
		}
	}

	return weight;
}










