// ===============================================================================
//						     PHYSX SDK TRAINING PROGRAMS
//						     LESSON 101 - BOX ON A PLANE
//
//				           Written by Bob Schade, 10-15-05
// ===============================================================================

#include <NxCooking.h>
#include "Main.h"
#include "UpdateTime.h"

#include "BSpline.h"

// raytracer support (uncomment it to render with OpenGl)
#include "raytracer/opengl2ray.h"

// Physics SDK globals
NxPhysicsSDK*     gPhysicsSDK = NULL;
NxScene*          gScene = NULL;
DebugRenderer     gDebugRenderer;
NxVec3            gDefaultGravity(0,-9.8,0);
 
// Display globals
int gMainHandle;
int mx = 0;
int my = 0;

// Camera globals
float gCameraAspectRatio = 1;
NxVec3 gCameraPos(22.8,36.5,-6.9);
NxVec3 gCameraForward(-0.5,-0.8,0.3);
NxVec3 gCameraRight(-1,0,0);
const NxReal gCameraSpeed = 0.15;

// Force globals
NxVec3 gForceVec(0,0,0);
NxReal gForceStrength = 150;
bool bForceMode = true;

// Keyboard globals
#define MAX_KEYS 256
bool gKeys[MAX_KEYS];

// Simulation globals
bool bPause = false;
bool bShadows = true;
bool bDebugWireframeMode = false;

// Actor globals
NxActor* groundPlane = NULL;
NxActor* box = NULL;

// Focus actor
NxActor* gSelectedActor = NULL;

NxActor* CreateBox(NxReal x, NxReal y, NxReal z);
NxActor* CreateSphere(NxReal x, NxReal y, NxReal z);
NxActor* CreateSphere(const NxVec3& pos, const NxReal radius, const NxReal density);

// BSpline for comera path
double cameraArrayPath[10000][3];
double viewArrayPath[10000][3];
double timeArray[10000][3];

bool simulation = false;
double simulationInterval;
double simulationDeltaT;

// screen resolution
int scene_width  = 1024;
int scene_height = 576;

void PrintControls()
{
	printf("\n Flight Controls:\n ----------------\n w = forward, s = back\n a = strafe left, d = strafe right\n q = up, z = down\n");
    printf("\n Force Controls:\n ---------------\n i = +z, k = -z\n j = +x, l = -x\n u = +y, m = -y\n");
	printf("\n Miscellaneous:\n --------------\n p = Pause\n r = Select Next Actor\n f = Toggle Force Mode\n b = Toggle Debug Wireframe Mode\n x = Toggle Shadows\n t = Move Focus Actor to (0,5,0)\n");
}

void startSimulation() {
	simulation = true;
	simulationInterval = 0.0;
	simulationDeltaT = 0.033;
	ResetNx();
}

NxVec3 ApplyForceToActor(NxActor* actor, const NxVec3& forceDir, const NxReal forceStrength)
{
	NxVec3 forceVec = forceStrength*forceDir;
	actor->addForce(forceVec);
	return forceVec;
}

void ProcessCameraKeys()
{
	// Process camera keys
	for (int i = 0; i < MAX_KEYS; i++)
	{	
		if (!gKeys[i])  { continue; }

		switch (i)
		{
			// Camera controls
			case 'w':{ gCameraPos += gCameraForward*gCameraSpeed; break; }
			case 's':{ gCameraPos -= gCameraForward*gCameraSpeed; break; }
			case 'a':{ gCameraPos -= gCameraRight*gCameraSpeed;   break; }
			case 'd':{ gCameraPos += gCameraRight*gCameraSpeed;   break; }
			case 'y':{ gCameraPos -= NxVec3(0,1,0)*gCameraSpeed;  break; }
			case 'q':{ gCameraPos += NxVec3(0,1,0)*gCameraSpeed;  break; }
		}
	}
}

void SetupCamera()
{
	// Setup camera
	glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluPerspective(60.0f, gCameraAspectRatio, 1.0f, 10000.0f);

	if (!simulation) {
		gluLookAt(gCameraPos.x,gCameraPos.y,gCameraPos.z,gCameraPos.x + gCameraForward.x, gCameraPos.y + gCameraForward.y, gCameraPos.z + gCameraForward.z, 0.0f, 1.0f, 0.0f);
	}
	else {
		simulationInterval += 1.0/16.0 * 0.01;

		if (simulationInterval > 1.0) {
			simulation = false;
		}
		else {
			int step = simulationInterval * 10000.0;

			gCameraPos.x = cameraArrayPath[step][0];
			gCameraPos.y = cameraArrayPath[step][1];
			gCameraPos.z = cameraArrayPath[step][2];

			gCameraForward.x = viewArrayPath[step][0] - gCameraPos.x;
			gCameraForward.y = viewArrayPath[step][1] - gCameraPos.y;
			gCameraForward.z = viewArrayPath[step][2] - gCameraPos.z;

			simulationDeltaT = timeArray[step][0];

			//printf("%d (%f)\n", step, simulationDeltaT);
		}

		gluLookAt(gCameraPos.x,gCameraPos.y,gCameraPos.z,gCameraPos.x + gCameraForward.x, gCameraPos.y + gCameraForward.y, gCameraPos.z + gCameraForward.z, 0.0f, 1.0f, 0.0f);
	}

	//printf("%f, %f, %f\n", gCameraPos.x,gCameraPos.y,gCameraPos.z);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}

void RenderActors(bool shadows)
{
    // Render all the actors in the scene
    int nbActors = gScene->getNbActors();
    NxActor** actors = gScene->getActors();
    while (nbActors--)
    {
        NxActor* actor = *actors++;
        DrawActor(actor);

        // Handle shadows
        if (shadows)
        {
			DrawActorShadow(actor);
        }
    }
}

void DrawForce(NxActor* actor, NxVec3& forceVec, const NxVec3& color) {
	// Draw only if the force is large enough
	NxReal force = forceVec.magnitude();
	if (force < 0.1f)  return;

	forceVec = 3*forceVec/force;

	NxVec3 pos = actor->getCMassGlobalPosition();
	DrawArrow(pos, pos + forceVec, color);
}

void ProcessForceKeys() {
	// Process force keys
	for (int i = 0; i < MAX_KEYS; i++) {	
		if (!gKeys[i])  { continue; }

		switch (i) {
			// Force controls
			case 'i': { gForceVec = ApplyForceToActor(box,NxVec3(0,0,5),gForceStrength); break; }
			case 'k': { gForceVec = ApplyForceToActor(box,NxVec3(0,0,-5),gForceStrength); break; }
			case 'j': { gForceVec = ApplyForceToActor(box,NxVec3(5,0,0),gForceStrength); break; }
			case 'l': { gForceVec = ApplyForceToActor(box,NxVec3(-5,0,0),gForceStrength); break; }
			case 'u': { gForceVec = ApplyForceToActor(box,NxVec3(0,5,0),gForceStrength); break; }
			case 'm': { gForceVec = ApplyForceToActor(box,NxVec3(0,-5,0),gForceStrength); break; }

		    // Return box to (0,5,0)
		    case 't': { box->setGlobalPosition(NxVec3(0,5,0)); break; }		}
	}
}

NxReal getDeltaTime(bool save) {
    NxReal deltaTime;
#ifndef LINUX
    static __int64 gTime,gLastTime;
    __int64 freq;
    QueryPerformanceCounter((LARGE_INTEGER *)&gTime);  // Get current count
    QueryPerformanceFrequency((LARGE_INTEGER *)&freq); // Get processor freq
    deltaTime = (double)(gTime - gLastTime)/(double)freq;
    gLastTime = gTime;
#else
	struct timeval tv;
	static struct timeval lasttv = { 0 , 0 };
	if (lasttv.tv_usec == 0 && lasttv.tv_sec == 0)
		gettimeofday(&lasttv, NULL);
	gettimeofday(&tv, NULL);
	deltaTime = (tv.tv_usec - lasttv.tv_usec)/1000000.f
		+ (tv.tv_sec - lasttv.tv_sec);
	lasttv = tv;
#endif
    return deltaTime;
}

void ProcessActionKeys() {
#ifndef LINUX
	static __int64 gTime,gLastTime;
	__int64 freq;

    QueryPerformanceCounter((LARGE_INTEGER *)&gTime);  // Get current count
    QueryPerformanceFrequency((LARGE_INTEGER *)&freq); // Get processor freq

    NxReal deltaTime = (double)(gTime - gLastTime)/(double)freq;
#else
	struct timeval tv;
	static struct timeval lasttv = { 0 , 0 };
	if (lasttv.tv_usec == 0 && lasttv.tv_sec == 0)
		gettimeofday(&lasttv, NULL);
	gettimeofday(&tv, NULL);
	NxReal deltaTime = (tv.tv_usec - lasttv.tv_usec)/1000000.f
		+ (tv.tv_sec - lasttv.tv_sec);
#endif

	if (deltaTime < 0.25) {
		return;
	}
	else {
#ifndef LINUX
		gLastTime = gTime;
#else
		lasttv = tv;
#endif
	}

	// Process force keys
	for (int i = 0; i < MAX_KEYS; i++) {	
		if (!gKeys[i])  { continue; }

		switch (i) {
			case ' ':
			 	box = CreateSphere(80, 4, 11);
				ApplyForceToActor(box, NxVec3(-2500,100,0),gForceStrength);
				break;
			case 13: // return key
				box = CreateSphere(NxVec3(gCameraPos.x, gCameraPos.y-2.5, gCameraPos.z), 0.5, 10);
				ApplyForceToActor(box, NxVec3(gCameraForward.x * 200,gCameraForward.y * 200,gCameraForward.z * 200),gForceStrength);
				break;
		}
	}
}

void ProcessInputs() {
    ProcessForceKeys();
	ProcessActionKeys();

    // Show debug wireframes
	if (bDebugWireframeMode)
	{
		if (gScene)  gDebugRenderer.renderData(*gScene->getDebugRenderable());
	}
}

void RenderCallback()
{
    // Clear buffers
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    ProcessCameraKeys();
	SetupCamera();

    if (gScene && !bPause)
	{
		GetPhysicsResults();
        ProcessInputs();
		StartPhysics();
	}
	
 	RenderActors(bShadows);

	DrawForce(box, gForceVec, NxVec3(1,1,0));
	gForceVec = NxVec3(0,0,0);

	// dump to bmp
	static int frame = 0;
	char filename[30];
	sprintf(filename, "image_openGl%05d.bmp", frame);
	frame++;

    glFlush();
    glutSwapBuffers();
}

void ReshapeCallback(int width, int height)
{
	scene_height = height;
	scene_width  = width;

    glViewport(0, 0, width, height);
    gCameraAspectRatio = float(width)/float(height);
	
}

void IdleCallback()
{
    glutPostRedisplay();
}

void KeyboardCallback(unsigned char key, int x, int y)
{
	gKeys[key] = true;

	switch (key)
	{
		case 'r': { break; }
		default: { break; }
	}
}

void KeyboardUpCallback(unsigned char key, int x, int y)
{
	gKeys[key] = false;

	switch (key)
	{
		case 'p': { bPause = !bPause; UpdateTime(); break; }
		case 'x': { bShadows = !bShadows; break; }
		case 'b': { bDebugWireframeMode = !bDebugWireframeMode; break; }
		case 27 : { exit(0); break; }
		default : { break; }
	}
}

void SpecialCallback(int key, int x, int y)
{
	switch (key)
    {
		// Reset PhysX
		case GLUT_KEY_F10:
			ResetNx();
			return; 
		case GLUT_KEY_F5:
			startSimulation();
			break;
	}
}

void MouseCallback(int button, int state, int x, int y)
{
    mx = x;
    my = y;
}

void MotionCallback(int x, int y)
{
    int dx = mx - x;
    int dy = my - y;
    
    gCameraForward.normalize();
    gCameraRight.cross(gCameraForward,NxVec3(0,1,0));

    NxQuat qx(NxPiF32 * dx * 20 / 180.0f, NxVec3(0,1,0));
    qx.rotate(gCameraForward);
    NxQuat qy(NxPiF32 * dy * 20 / 180.0f, gCameraRight);
    qy.rotate(gCameraForward);

    mx = x;
    my = y;
}

void ExitCallback()
{
	ReleaseNx();
}

void InitGlut(int argc, char** argv)
{
    glutInit(&argc, argv);
    glutInitWindowSize(scene_width, scene_height);
    glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH);
    gMainHandle = glutCreateWindow("Rigid Body Simulation");
    glutSetWindow(gMainHandle);
    glutDisplayFunc(RenderCallback);
    glutReshapeFunc(ReshapeCallback);
    glutIdleFunc(IdleCallback);
    glutKeyboardFunc(KeyboardCallback);
    glutKeyboardUpFunc(KeyboardUpCallback);
	glutSpecialFunc(SpecialCallback);
    glutMouseFunc(MouseCallback);
    glutMotionFunc(MotionCallback);
	MotionCallback(0,0);

	glutExitFunc(ExitCallback);

    // Setup default render states
    glClearColor(0.0f, 0.0f, 0.0f, 1.0);
    glEnable(GL_DEPTH_TEST);
    glEnable(GL_COLOR_MATERIAL);
    glEnable(GL_CULL_FACE);

    // Setup lighting
    glEnable(GL_LIGHTING);
    float AmbientColor[]    = { 0.0f, 0.1f, 0.2f, 0.0f };         glLightfv(GL_LIGHT0, GL_AMBIENT, AmbientColor);
    float DiffuseColor[]    = { 0.2f, 0.2f, 0.2f, 0.0f };         glLightfv(GL_LIGHT0, GL_DIFFUSE, DiffuseColor);
    float SpecularColor[]   = { 0.5f, 0.5f, 0.5f, 0.0f };         glLightfv(GL_LIGHT0, GL_SPECULAR, SpecularColor);
    float Position[]        = { 100.0f, 100.0f, -400.0f, 1.0f };  glLightfv(GL_LIGHT0, GL_POSITION, Position);
    glEnable(GL_LIGHT0);

	InitShapeDisplayLists();
}

NxActor* CreateGroundPlane()
{
    // Create a plane with default descriptor
    NxPlaneShapeDesc planeDesc;
    NxActorDesc actorDesc;
    actorDesc.shapes.pushBack(&planeDesc);
    return gScene->createActor(actorDesc);
}

NxActor* CreateBox(NxReal x, NxReal y, NxReal z) {
	// Add a single-shape actor to the scene
	NxActorDesc actorDesc;
	NxBodyDesc bodyDesc;

	// The actor has one shape, a box, 1m on a side
	NxBoxShapeDesc boxDesc;
	boxDesc.dimensions.set(0.5,0.5,0.5);
	actorDesc.shapes.pushBack(&boxDesc);

	actorDesc.body         = &bodyDesc;
	actorDesc.density      = 10;
	actorDesc.globalPose.t = NxVec3(x, y, z);
	return gScene->createActor(actorDesc);	
}

NxActor* CreateSphere(NxReal x, NxReal y, NxReal z) {
	// Add a single-shape actor to the scene
	NxActorDesc actorDesc;
	NxBodyDesc bodyDesc;

	// The actor has one shape, a sphere
	NxSphereShapeDesc sphereDesc;
	sphereDesc.radius = 1.2;
	sphereDesc.localPose.t = NxVec3(0,1.2,0);
	actorDesc.shapes.pushBack(&sphereDesc);

	actorDesc.body = &bodyDesc;
	actorDesc.density = 10.0;
	actorDesc.globalPose.t = NxVec3(x,y,z);
	return gScene->createActor(actorDesc);
}

NxActor* CreateSphere(const NxVec3& pos, const NxReal radius, const NxReal density) {
	// Add a single-shape actor to the scene
	NxActorDesc actorDesc;
	NxBodyDesc bodyDesc;

	// The actor has one shape, a sphere
	NxSphereShapeDesc sphereDesc;
	sphereDesc.radius = radius;
	sphereDesc.localPose.t = NxVec3(0,radius,0);
	actorDesc.shapes.pushBack(&sphereDesc);

	if (density)
	{
		actorDesc.body = &bodyDesc;
		actorDesc.density = density;
	}
	else
	{
		actorDesc.body = NULL;
	}
	actorDesc.globalPose.t = pos;
	return gScene->createActor(actorDesc);	
}

/*
NxActor* CreateConcaveObject(const char* filename)
{
	// Add a single-shape actor to the scene
	NxActorDesc actorDesc;
	NxBodyDesc bodyDesc;

	// Load Wavefront file
	WavefrontObj wo;
	unsigned int tcount = wo.loadObj(filename);

	// Create descriptor for triangle mesh
    NxTriangleMeshDesc concaveDesc;
	concaveDesc.numVertices = wo.mVertexCount;
    concaveDesc.pointStrideBytes = sizeof(NxVec3);
	concaveDesc.points = wo.mVertices;
	concaveDesc.numTriangles = wo.mTriCount;
	concaveDesc.triangles = wo.mIndices;
    concaveDesc.triangleStrideBytes = 3 * sizeof(NxU32);
    concaveDesc.flags = 0;

	//cooking is a slow process and should generally be done offline.
    NxInitCooking();

    MemoryWriteBuffer   write;
    NxCookTriangleMesh(concaveDesc, write);
    MemoryReadBuffer    read(write.data);

    // Novodex::DoneSDK();

    // The actor has one shape, a triangle mesh
    NxTriangleMeshShapeDesc concaveShapeDesc;
    concaveShapeDesc.meshData = gPhysicsSDK->createTriangleMesh(read);  //gPhysicsSDK->createTriangleMesh(concaveDesc.);
    concaveShapeDesc.localPose.t = NxVec3(4,4,4);
    actorDesc.shapes.pushBack(&concaveShapeDesc);

    if (10)
    {
		actorDesc.body = (NxBodyDesc*)&concaveShapeDesc;
        actorDesc.density = 10;
    }
    else
    {
        actorDesc.body = NULL;
    }

    actorDesc.globalPose.t = NxVec3(5, 5, 5); //pos;
    return gScene->createActor(actorDesc);
} */


void InitNx()
{
    // Create the physics SDK
    gPhysicsSDK = NxCreatePhysicsSDK(NX_PHYSICS_SDK_VERSION);
    if (!gPhysicsSDK)
		return;

	// Set the physics parameters
	gPhysicsSDK->setParameter(NX_SKIN_WIDTH, 0.02);

	// Set the debug visualization parameters
	gPhysicsSDK->setParameter(NX_VISUALIZATION_SCALE, 1);
	gPhysicsSDK->setParameter(NX_VISUALIZE_COLLISION_SHAPES, 1);
	gPhysicsSDK->setParameter(NX_VISUALIZE_ACTOR_AXES, 1);

    // Create the scene
    NxSceneDesc sceneDesc;
    sceneDesc.gravity               = gDefaultGravity;
    sceneDesc.broadPhase            = NX_BROADPHASE_COHERENT;
    sceneDesc.collisionDetection    = true;
    gScene = gPhysicsSDK->createScene(sceneDesc);

	// Create the default material
	NxMaterial* defaultMaterial = gScene->getMaterialFromIndex(0); 
	defaultMaterial->setRestitution(0.5);
	defaultMaterial->setStaticFriction(0.5);
	defaultMaterial->setDynamicFriction(0.5);

	// Create the objects in the scene
	groundPlane = CreateGroundPlane();

	for(int j=0; j<10; ++j) {
		for(int i=0; i<23-j; ++i) {
			box = CreateBox(0, 1.0 + j, j*0.5f + i);
		}
	}

	for(int j=0; j<10; ++j) {
		for(int i=0; i<19-j; ++i) {
			box = CreateBox(-10, 1.0 + j, 2.0 + j*0.5f + i);
		}
	}

	// Get the current time
	UpdateTime();

	// Start the first frame of the simulation
	if (gScene)  StartPhysics();

	// make some steps
	for(int i=0; i<100; ++i) {
		if (gScene && !bPause) {
			GetPhysicsResults();
			StartPhysics();
		}
	}

	// first get some physics result
	GetPhysicsResults();

	// create ball
	if (simulation) {
		// ball ball fly
 		box = CreateSphere(80, 4, 11);
		ApplyForceToActor(box, NxVec3(-2500,100,0),gForceStrength);

		gScene->setTiming(1.0/535.0, 8, NX_TIMESTEP_FIXED);
	}

	// get physics back online
	if (gScene)  StartPhysics();

	//NxReal maxTimestep;
	//NxU32  maxIter;
	//NxTimeStepMethod method;
	//NxU32 subStebs;

	//gScene->getTiming(maxTimestep, maxIter, method, &subStebs);
}

void ReleaseNx() {
    if (gScene) {
		GetPhysicsResults();  // Make sure to fetchResults() before shutting down
		gPhysicsSDK->releaseScene(*gScene);
	}
	if (gPhysicsSDK)  gPhysicsSDK->release();
}

void ResetNx()
{
	ReleaseNx();
	InitNx();
}

void StartPhysics()
{
#ifndef _OPENGL2RAY
//	static float time = 0.0f;

	// Update the time step
	NxReal deltaTime = UpdateTime();

	// Start collision and dynamics for delta time since the last frame
	if (!simulation) { 
		gScene->simulate(deltaTime);
//		time += deltaTime;
	}
	else {
		gScene->simulate(simulationDeltaT);
//		time += simulationDeltaT;
	}
	gScene->flushStream();

//	time += 0.033;
//	printf("%f\n", time);
#else
	// Update the time step
	NxReal deltaTime = UpdateTime();

	// Start collision and dynamics for delta time since the last frame
	if (!simulation) { 
		gScene->simulate(deltaTime);
	}
	else {
		gScene->simulate(simulationDeltaT);
	}
	gScene->flushStream();
#endif
}

void GetPhysicsResults() {
	// Get results from gScene->simulate(deltaTime)
	while (!gScene->fetchResults(NX_RIGID_BODY_FINISHED, false));
}

///////////////////////////////////////////////////////////////////////////////
//////////////////////////////FROM HERE ON:////////////////////////////////////
/////////////////EVERYTHING NEEDED FOR RAYTRCING THIS SCENE////////////////////
///////////////////////////////////////////////////////////////////////////////

void init_opengl_scene(void) {
	PrintControls();
//    InitNx();
	startSimulation();

//	box = CreateSphere(80, 4, 11);
//	ApplyForceToActor(box, NxVec3(-2500,100,0),gForceStrength);

	// shot    frame
	// shot 0:  15
	// shot 1:  30
	// shot 2:  50

	//for(int i=0; i<2; ++i) {
	//	if (gScene && !bPause) {
	//		GetPhysicsResults();
	//		StartPhysics();
	//	}
	//}
}

void uninit_opengl_scene(void) {
	ReleaseNx();
}

void RenderShape(NxShape *shape) {
	NxMat34 pose = shape->getGlobalPose();
	NxVec3 pos = shape->getGlobalPosition();

    switch(shape->getType()) {
		case NX_SHAPE_PLANE: {
			float smat[3] = {0.1,0.2,0.3};
			float sk[4]   = {0.8,0.5,0.0,0.0};

			float q[3] = {0.0,0.0,0.0};
			float n[3] = {0.0,1.0,0.0};

			Primitive *prim = new Infplane(q, n, smat, sk);
			primitives.push_back(prim);

			//primitives.push_back(new Sphere(1.0,spos1,smat,sk));
			//glDisable(GL_LIGHTING);
			//glColor3f(0.1f, 0.2f, 0.3f);
			//pose.t.y -= 0.1f;
			//SetupGLMatrix(pose.t, pose.M);
			//glScaled(1024,0,1024);
			//glCallList(dispListPlane);
			//glColor3f(1.0f, 1.0f, 1.0f);
			//glEnable(GL_LIGHTING);
			break;
		}
		case NX_SHAPE_BOX: {
			NxVec3  pos    = pose.t;
			NxMat33 orient = pose.M;

			float glmat[16];	//4x4 column major matrix for OpenGL.
			orient.getColumnMajorStride4(&(glmat[0]));
			//pos.get(&(glmat[12]));

			//clear the elements we don't need:
			//glmat[3] = glmat[7] = glmat[11] = 0.0f;
			//glmat[15] = 1.0f;

			float mult[3][3] = {glmat[ 0], glmat[ 1], glmat[ 2],
				                glmat[ 4], glmat[ 5], glmat[ 6], 
								glmat[ 8], glmat[ 9], glmat[10]};


			float p[3]  = {pos.x, pos.y, pos.z};

			float q1[3] = {+0.5,+0.5,+0.5};
			float q2[3] = {+0.5,+0.5,-0.5};
			float q3[3] = {+0.5,-0.5,+0.5};
			float q4[3] = {+0.5,-0.5,-0.5};
			float q5[3] = {-0.5,+0.5,+0.5};
			float q6[3] = {-0.5,+0.5,-0.5};
			float q7[3] = {-0.5,-0.5,+0.5};
			float q8[3] = {-0.5,-0.5,-0.5};


			float no1[3] = { 0.0, 1.0, 0.0};
			float no2[3] = { 0.0, 0.0, 1.0};
			float no3[3] = { 1.0, 0.0, 0.0};
			float no4[3] = { 0.0,-1.0, 0.0};
			float no5[3] = { 0.0, 0.0,-1.0};
			float no6[3] = {-1.0, 0.0, 0.0};

			float tmat[3] = {0.1,0.2,0.3};
			float tk[4] = {1.2,0.3,0.0,1.0};

			Triangle *t1 = new Triangle(q1,q2,q3,no3,no3,no3,tmat,tk);
			Triangle *t2 = new Triangle(q2,q3,q4,no3,no3,no3,tmat,tk);
			t1->transform(mult);
			t2->transform(mult);
			t1->translate(p);
			t2->translate(p);
			primitives.push_back(t1);
			primitives.push_back(t2);

			t1 = new Triangle(q7,q5,q1,no2,no2,no2,tmat,tk);
			t2 = new Triangle(q7,q1,q3,no2,no2,no2,tmat,tk);
			t1->transform(mult);
			t2->transform(mult);
			t1->translate(p);
			t2->translate(p);
			primitives.push_back(t1);
			primitives.push_back(t2);

			t1 = new Triangle(q6,q2,q8,no5,no5,no5,tmat,tk);
			t2 = new Triangle(q8,q2,q4,no5,no5,no5,tmat,tk);
			t1->transform(mult);
			t2->transform(mult);
			t1->translate(p);
			t2->translate(p);
			primitives.push_back(t1);
			primitives.push_back(t2);

			t1 = new Triangle(q5,q6,q2,no1,no1,no1,tmat,tk);
			t2 = new Triangle(q5,q2,q1,no1,no1,no1,tmat,tk);
			t1->transform(mult);
			t2->transform(mult);
			t1->translate(p);
			t2->translate(p);
			primitives.push_back(t1);
			primitives.push_back(t2);

			t1 = new Triangle(q6,q5,q7,no6,no6,no6,tmat,tk);
			t2 = new Triangle(q7,q6,q8,no6,no6,no6,tmat,tk);
			t1->transform(mult);
			t2->transform(mult);
			t1->translate(p);
			t2->translate(p);
			primitives.push_back(t1);
			primitives.push_back(t2);

			t1 = new Triangle(q7,q8,q4,no4,no4,no4,tmat,tk);
			t2 = new Triangle(q7,q4,q3,no4,no4,no4,tmat,tk);
			t1->transform(mult);
			t2->transform(mult);
			t1->translate(p);
			t2->translate(p);
			primitives.push_back(t1);
			primitives.push_back(t2);

			//SetupGLMatrix(pose.t, pose.M);
			//NxVec3 boxDim = box->isBox()->getDimensions();
			//glScaled(boxDim.x, boxDim.y, boxDim.z);
			//glCallList(dispListBox);
			break;
		}
		case NX_SHAPE_SPHERE: {
			NxVec3 pos    = pose.t;

			float p[3]    = {pos.x, pos.y, pos.z};
			float smat[3] = {0.1,0.2,0.3};
			float sk[4]   = {0.8,0.5,0.0,0.0};

			primitives.push_back(new Sphere(1.2, p, smat, sk));

			//SetupGLMatrix(pose.t, pose.M);
			//NxReal r = sphere->isSphere()->getRadius();
			//glScaled(r,r,r);
			//glCallList(dispListSphere);
			break;
		}
	}
}

void RenderRaytracerScene() {
    // Render all the actors in the scene
    int nbActors = gScene->getNbActors();
    NxActor** actors = gScene->getActors();
    while(nbActors--) {
        NxActor* actor = *actors++;

		// RenderActor
		NxShape*const* shapes = actor->getShapes();
		NxU32 nShapes = actor->getNbShapes();
		while(nShapes--) {
			RenderShape(shapes[nShapes]);
		}
	}
}

void redraw(void) {
//	rayPerspective(60, 1.0f);
//	rayLookAt(20.0f,15.0f,20.0f,
//			  0.0f, -10.0f, 0.0f,
//			  0.0f,   1.0f, 0.0f);

	// Setup camera
	rayPerspective(60, 1.0f);

	if (!simulation) {
		rayLookAt(gCameraPos.x,gCameraPos.y,gCameraPos.z,gCameraPos.x + gCameraForward.x, gCameraPos.y + gCameraForward.y, gCameraPos.z + gCameraForward.z, 0.0f, 1.0f, 0.0f);
	}
	else {
		simulationInterval += 1.0/16.0 * 0.01;

		if (simulationInterval > 1.0) {
			simulation = false;
		}
		else {
			int step = simulationInterval * 10000.0;

			gCameraPos.x = cameraArrayPath[step][0];
			gCameraPos.y = cameraArrayPath[step][1];
			gCameraPos.z = cameraArrayPath[step][2];

			gCameraForward.x = viewArrayPath[step][0];
			gCameraForward.y = viewArrayPath[step][1];
			gCameraForward.z = viewArrayPath[step][2];

			simulationDeltaT = timeArray[step][0];
		}

		rayLookAt(gCameraPos.x,gCameraPos.y,gCameraPos.z,gCameraForward.x, gCameraForward.y, gCameraForward.z, 0.0f, 1.0f, 0.0f);
	}


    if (gScene && !bPause) {
		GetPhysicsResults();
		StartPhysics();
	}

	NxVec3 pos(0,0,0);

    // Display scene
 	RenderRaytracerScene();

	float p[3]  = {0,2,0};
	float smat[3] = {0.1,0.2,0.3};
	float sk[4]   = {0.8,0.5,0.0,0.0};
}

int main(int argc, char** argv)
{
	BSpline cameraPath;
	BSpline viewPath;
	BSpline time;

    // Display scene
	cameraPath.addControlPoint(22.8,36.5,-6.9);
	cameraPath.addControlPoint(16,20,-2);
	cameraPath.addControlPoint(-5,18,-10);
	cameraPath.addControlPoint(-25,18,-2);
	cameraPath.addControlPoint(-22,13,30);
	cameraPath.addControlPoint(6,11,32);
	cameraPath.addControlPoint(11,5,0);
	cameraPath.addControlPoint(-2,7,-4);
	cameraPath.addControlPoint(-23,10,-8);
	cameraPath.addControlPoint(-32,12,-10);
	cameraPath.addControlPoint(-62,14,-25);

	cameraPath.bspline(4, cameraArrayPath, 10000);

    // Display scene
	viewPath.addControlPoint(9.5,3,10.5);
	viewPath.addControlPoint(-11,2,10.5);

	viewPath.bspline(2, viewArrayPath, 10000);

	// timeflow
	time.addControlPoint(0.0150,    0.0,0.0);
	time.addControlPoint(0.0005, 1000.0,0.0);
	time.addControlPoint(0.0005, 1000.0,0.0);
	time.addControlPoint(0.0000, 1250.0,0.0);
	time.addControlPoint(0.0000, 5000.0,0.0);
	time.addControlPoint(0.0010, 4000.0,0.0);
	time.addControlPoint(0.0010, 4000.0,0.0);
	time.addControlPoint(0.0333, 6000.0,0.0);
	time.addControlPoint(0.0333, 8000.0,0.0);
	time.addControlPoint(0.0333,10000.0,0.0);

	time.bspline(3, timeArray, 10000);

#ifndef _OPENGL2RAY
	PrintControls();
    InitGlut(argc, argv);
    InitNx();
    glutMainLoop();
	ReleaseNx();
	return 0;
#else
	opengl2ray();
#endif
}