#include <stdio.h>
#include <vector>

#include "primitive.h"
#include "triangle.h"

#pragma once

using namespace std;

/* defines */
#define GLM_PI       3.14159265358979323846
#define GLM_NONE     (0)			/* render with only vertices */
#define GLM_FLAT     (1 << 0)		/* render with facet normals */
#define GLM_SMOOTH   (1 << 1)		/* render with vertex normals */
#define GLM_TEXTURE  (1 << 2)		/* render with texture coords */
#define GLM_COLOR    (1 << 3)		/* render with colors */
#define GLM_MATERIAL (1 << 4)		/* render with materials */


/* structs */

/* rayGLMmaterial: Structure that defines a material in a model. 
*/
typedef struct _rayGLMmaterial
{
	char* name;					/* name of material */
	float diffuse[4];			/* diffuse component */
	float ambient[4];			/* ambient component */
	float specular[4];			/* specular component */
	float emmissive[4];			/* emmissive component */
	float shininess;			/* specular exponent */
	char* texture_file;			/* file_name of texture */
} rayGLMmaterial;

/* rayGLMtriangle: Structure that defines a triangle in a model.
*/
typedef struct {
	unsigned int vindices[3];	/* array of triangle vertex indices */
	unsigned int nindices[3];	/* array of triangle normal indices */
	unsigned int tindices[3];	/* array of triangle texcoord indices*/
	unsigned int findex;		/* index of triangle facet normal */
} rayGLMtriangle;

/* rayGLMgroup: Structure that defines a group in a model.
*/
typedef struct _rayGLMgroup {
	char*			name;			/* name of this group */
	unsigned int	numtriangles;	/* number of triangles in this group */
	unsigned int*	triangles;		/* array of triangle indices */
	unsigned int	material;		/* index to material for group */
	struct _rayGLMgroup* next;		/* pointer to next group in model */
} rayGLMgroup;

/* rayGLMmodel: Structure that defines a model.
*/
typedef struct {
	char*    pathname;				/* path to this model */
	char*    mtllibname;			/* name of the material library */

	unsigned int   numvertices;		/* number of vertices in model */
	float* vertices;				/* array of vertices  */

	unsigned int   numnormals;		/* number of normals in model */
	float* normals;					/* array of normals */

	unsigned int   numtexcoords;	/* number of texcoords in model */
	float* texcoords;				/* array of texture coordinates */

	unsigned int   numfacetnorms;	/* number of facetnorms in model */
	float* facetnorms;				/* array of facetnorms */

	unsigned int   numtriangles;	/* number of triangles in model */
	rayGLMtriangle* triangles;		/* array of triangles */

	unsigned int   nummaterials;	/* number of materials in model */
	rayGLMmaterial* materials;		/* array of materials */

	unsigned int    numgroups;		/* number of groups in model */
	rayGLMgroup*    groups;			/* linked list of groups */

	float position[3];				/* position of the model */

} rayGLMmodel;


typedef struct kdNode {
	kdNode *left;
	kdNode *right;

	vector<Primitive *> prim;

	float xmin, xmax;
	float ymin, ymax;
	float zmin, zmax;

	float n1[3]; float d1;
	float n2[3]; float d2;
	float n3[3]; float d3;
	float n4[3]; float d4;
	float n5[3]; float d5;
	float n6[3]; float d6;
};

class rayObjectLoader : public Primitive {
public:
	rayObjectLoader(const char *path) {
		model = glmReadOBJ(path);
	}

	kdNode* create_kdTree(int dir, vector<Triangle *> triangles);

	// virtual functions (must be declared in descendend class)
	virtual float t(float *p,float *d);
	virtual float tex(float *p,float *d, float *uv);
	virtual float cut_dist(float *p,float *d)       { return cut_prim->cut_dist(p, d); };
	virtual void transform(float m[3][3]);
	virtual void translate(float *t);
	virtual void normal(float *hit, float *n)       { cut_prim->normal(hit, n); };
	virtual void partial_derivatives(float *uv, float *duv) { cut_prim->partial_derivatives(uv, duv); };
	virtual void material(float *m)                 { cut_prim->material(m); };
	virtual float get_kd(void)                      { return cut_prim->get_kd(); };
	virtual float get_ks(void)                      { return cut_prim->get_ks(); };
	virtual float get_kr(void)                      { return cut_prim->get_kr(); };
	virtual float get_index(void)                   { return cut_prim->get_index(); };
	virtual int get_texture_index(void)             { return cut_prim->get_texture_index(); };
	virtual int get_bump_index(void)				{ return cut_prim->get_bump_index(); };
	virtual bool textured(void)                     { return false; };
	virtual bool bump_mapping(void)                 { return false; };

private:
	float t_recursive(float *p, float *d, kdNode *node, float min);
	float tex_recursive(float *p, float *d, float *uv, kdNode *node, float min);

private:
	Primitive *cut_prim;
	kdNode    *kdTree;

	vector<Triangle *> objprim;

public:
	/* glmUnitize: "unitize" a model by translating it to the origin and
	* scaling it to fit in a unit cube around the origin.  Returns the
	* scalefactor used.
	*/
	float glmUnitize();

	/* glmDimensions: Calculates the dimensions (width, height, depth) of
	* a model.
	*
	* dimensions - array of 3 floats (float dimensions[3])
	*/
	void glmDimensions(float* dimensions);

	/* glmScale: Scales a model by a given amount.
	* 
	* scale - scalefactor (0.5 = half as large, 2.0 = twice as large)
	*/
	void glmScale(float scale);

	/* glmReverseWinding: Reverse the polygon winding for all polygons in
	* this model.  Default winding is counter-clockwise.  Also changes
	* the direction of the normals.
	*/
	void glmReverseWinding();

	/* glmFacetNormals: Generates facet normals for a model (by taking the
	* cross product of the two vectors derived from the sides of each
	* triangle).  Assumes a counter-clockwise winding.
	*/
	void glmFacetNormals();

	/* glmVertexNormals: Generates smooth vertex normals for a model.
	* First builds a list of all the triangles each vertex is in.  Then
	* loops through each vertex in the the list averaging all the facet
	* normals of the triangles each vertex is in.  Finally, sets the
	* normal index in the triangle for the vertex to the generated smooth
	* normal.  If the dot product of a facet normal and the facet normal
	* associated with the first triangle in the list of triangles the
	* current vertex is in is greater than the cosine of the angle
	* parameter to the function, that facet normal is not added into the
	* average normal calculation and the corresponding vertex is given
	* the facet normal.  This tends to preserve hard edges.  The angle to
	* use depends on the model, but 90 degrees is usually a good start.
	*
	* angle - maximum angle (in degrees) to smooth across
	*/
	void glmVertexNormals(float angle);

	/* glmLinearTexture: Generates texture coordinates according to a
	* linear projection of the texture map.  It generates these by
	* linearly mapping the vertices onto a square.
	*/
	void glmLinearTexture();

	/* glmSpheremapTexture: Generates texture coordinates according to a
	* spherical projection of the texture map.  Sometimes referred to as
	* spheremap, or reflection map texture coordinates.  It generates
	* these by using the normal to calculate where that vertex would map
	* onto a sphere.  Since it is impossible to map something flat
	* perfectly onto something spherical, there is distortion at the
	* poles.  This particular implementation causes the poles along the X
	* axis to be distorted.
	*/
	void glmSpheremapTexture();

	/* glmDelete: Deletes a rayGLMmodel structure.
	*/
	void glmDelete();

	/* glmReadOBJ: Reads a model description from a Wavefront .OBJ file.
	* Returns a pointer to the created object which should be free'd with
	* glmDelete().
	*
	* filename - name of the file containing the Wavefront .OBJ format data.  
	*/
	rayGLMmodel*  glmReadOBJ(const char* filename);

	/* glmWriteOBJ: Writes a model description in Wavefront .OBJ format to
	* a file.
	*
	* filename - name of the file to write the Wavefront .OBJ format data to
	* mode     - a bitwise or of values describing what is written to the file
	*            GLM_NONE    -  write only vertices
	*            GLM_FLAT    -  write facet normals
	*            GLM_SMOOTH  -  write vertex normals
	*            GLM_TEXTURE -  write texture coords
	*            GLM_FLAT and GLM_SMOOTH should not both be specified.
	*/
	void glmWriteOBJ(char* filename, unsigned int mode);

	/* glmDraw: Renders the model to the current OpenGL context using the
	* mode specified.
	*
	* mode     - a bitwise OR of values describing what is to be rendered.
	*            GLM_NONE    -  render with only vertices
	*            GLM_FLAT    -  render with facet normals
	*            GLM_SMOOTH  -  render with vertex normals
	*            GLM_TEXTURE -  render with texture coords
	*            GLM_FLAT and GLM_SMOOTH should not both be specified.
	*/
	Primitive* draw(unsigned int mode);

	/* glmList: Generates and returns a display list for the model using
	* the mode specified.
	*
	* mode     - a bitwise OR of values describing what is to be rendered.
	*            GLM_NONE    -  render with only vertices
	*            GLM_FLAT    -  render with facet normals
	*            GLM_SMOOTH  -  render with vertex normals
	*            GLM_TEXTURE -  render with texture coords
	*            GLM_FLAT and GLM_SMOOTH should not both be specified.  
	*/
	Primitive* List(unsigned int mode);

	/* glmWeld: eliminate (weld) vectors that are within an epsilon of
	* each other.
	*
	* epsilon    - maximum difference between vertices
	*              ( 0.00001 is a good start for a unitized model)
	*
	*/
	void glmWeld(float epsilon);

private:
	rayGLMmodel* model;
	void cut_plane(float *p, float *d, float *n, float dx, float *cut);

	float _glmMax(float a, float b);

	/* _glmAbs: returns the absolute value of a float */
	float _glmAbs(float f);

	/* _glmDot: compute the dot product of two vectors
	*
	* u - array of 3 floats (float u[3])
	* v - array of 3 floats (float v[3])
	*/
	float _glmDot(float* u, float* v);

	/* _glmCross: compute the cross product of two vectors
	*
	* u - array of 3 floats (float u[3])
	* v - array of 3 floats (float v[3])
	* n - array of 3 floats (float n[3]) to return the cross product in
	*/
	void _glmCross(float* u, float* v, float* n);

	/* _glmNormalize: normalize a vector
	*
	* n - array of 3 floats (float n[3]) to be normalized
	*/
	void _glmNormalize(float* n);

	/* _glmEqual: compares two vectors and returns GL_TRUE if they are
	* equal (within a certain threshold) or GL_FALSE if not. An epsilon
	* that works fairly well is 0.000001.
	*
	* u - array of 3 floats (float u[3])
	* v - array of 3 floats (float v[3]) 
	*/
	bool _glmEqual(float* u, float* v, float epsilon);

	/* _glmWeldVectors: eliminate (weld) vectors that are within an
	* epsilon of each other.
	*
	* vectors    - array of float[3]'s to be welded
	* numvectors - number of float[3]'s in vectors
	* epsilon    - maximum difference between vectors 
	*
	*/
	float* _glmWeldVectors(float* vectors, unsigned int* numvectors, float epsilon);

	/* _glmFindGroup: Find a group in the model
	*/
	rayGLMgroup* _glmFindGroup(rayGLMmodel* model, char* name);

	/* _glmAddGroup: Add a group to the model
	*/
	rayGLMgroup* _glmAddGroup(rayGLMmodel* model, char* name);

	/* _glmFindGroup: Find a material in the model
	*/
	unsigned int _glmFindMaterial(rayGLMmodel* model, char* name);


	/* _glmDirName: return the directory given a path
	*
	* path - filesystem path
	*
	* The return value should be free'd.
	*/
	char* _glmDirName(char* path);

	/* _glmReadMTL: read a wavefront material library file
	*
	* model - properly initialized rayGLMmodel structure
	* name  - name of the material library
	*/
	void _glmReadMTL(rayGLMmodel* model, char* name);

	/* _glmWriteMTL: write a wavefront material library file
	*
	* model      - properly initialized rayGLMmodel structure
	* modelpath  - pathname of the model being written
	* mtllibname - name of the material library to be written
	*/
	void _glmWriteMTL(rayGLMmodel* model, char* modelpath, char* mtllibname);

	/* _glmFirstPass: first pass at a Wavefront OBJ file that gets all the
	* statistics of the model (such as #vertices, #normals, etc)
	*
	* model - properly initialized rayGLMmodel structure
	* file  - (fopen'd) file descriptor 
	*/
	void _glmFirstPass(rayGLMmodel* model, FILE* file);

	/* _glmSecondPass: second pass at a Wavefront OBJ file that gets all
	* the data.
	*
	* model - properly initialized rayGLMmodel structure
	* file  - (fopen'd) file descriptor 
	*/
	void _glmSecondPass(rayGLMmodel* model, FILE* file) ;

};