// rigid_bodies.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include <stdlib.h>
#include <fstream>
#include <iostream>
#include <vector>
#include "lp_lib.h"

using namespace std;

RIGID_BODY *rigidcube;

OBB *cubeobb;
OBB *groundobb;
OBB *walls[6];

vector<RIGID_BODY*> rigids;
int nbr_of_rigids = 0;

float dt = 0.0025;
float time = 0.0;

vector<contact*> separators;

vector<contact*> resting_contacts;
int resting_contacts_count;

bool colliding_contact_solved = false;

Vector3d force;

Vector3d MVMult(float A[3][3], Vector3d x){
	// Multiply Matrix A with vector x
	return Vector3d(Vector3d(A[0]).DotProduct(x),Vector3d(A[1]).DotProduct(x),Vector3d(A[2]).DotProduct(x));
}

Vector3d compute_ndot(contact *cont){
	// compute derivative of normal with respect to time
	if(cont->vf){
		return cont->b->w.CrossProduct(cont->n);
	}else{
		return Vector3d(0.0,0.0,0.0); // ?????
	}
}

void compute_b_vector(vector<contact*> contacts, int contactnbr, float **bvec){
	// compute vector needed to calculate forces on resting contacts 
	int i;
	for (i = 0; i < contactnbr; i++){
		contact *cont = contacts[i];
		RIGID_BODY *a = cont->a; RIGID_BODY *b = cont->b;
		Vector3d n = cont->n, ra = cont->p - a->xcm, rb = cont->p - b->xcm, 
		f_ext_a = a->F, f_ext_b = b->F, t_ext_a = a->torque, t_ext_b = b->torque;
		
		Vector3d a_ext_part = f_ext_a*a->invM + (MVMult(a->I,t_ext_a).CrossProduct(ra));
		Vector3d b_ext_part = f_ext_b*b->invM + (MVMult(b->I,t_ext_b).CrossProduct(rb));

		Vector3d a_vel_part = (a->w.CrossProduct(a->w.CrossProduct(ra))) + (MVMult(a->I,a->L.CrossProduct(a->w)).CrossProduct(ra));
		Vector3d b_vel_part = (b->w.CrossProduct(b->w.CrossProduct(rb))) + (MVMult(b->I,b->L.CrossProduct(b->w)).CrossProduct(rb));

		float k1 = n.DotProduct((a_ext_part + a_vel_part)-(b_ext_part + b_vel_part));
		Vector3d ndot = compute_ndot(cont);
		float k2 = ndot.DotProduct(a->vcm+a->w.CrossProduct(ra) - b->vcm+b->w.CrossProduct(rb))*2.0;
		(*bvec)[i] = k1+k2;
	}
}

float compute_aij(contact *ci, contact *cj){
	// compute entry aij of matrix a
	if((ci->a != cj->a)&&(ci->b != cj->b)&&(ci->a != cj->b)&&(ci->b != cj->a)){
		return 0.0;
	}

	RIGID_BODY *a = ci->a, *b = ci->b;
	Vector3d ni = ci->n, nj = cj->n, pi = ci->p, pj = cj->p, ra = pi - a->xcm, rb = pi - b->xcm;

	Vector3d force_on_a = Vector3d(0.0,0.0,0.0), torque_on_a = Vector3d(0.0,0.0,0.0);

	if (cj->a == ci->a){
		force_on_a = nj; torque_on_a = (pj - a->xcm).CrossProduct(nj);
	}else if (cj->b == ci->a){
		force_on_a = -nj; torque_on_a = (pj - a->xcm).CrossProduct(nj);
	}

	Vector3d force_on_b = Vector3d(0.0,0.0,0.0), torque_on_b = Vector3d(0.0,0.0,0.0);

	if (cj->a == ci->b){
		force_on_b = nj; torque_on_b = (pj - b->xcm).CrossProduct(nj);
	}else if (cj->b == ci->b){
		force_on_b = -nj; torque_on_b = (pj - b->xcm).CrossProduct(nj);
	}

	Vector3d a_linear = force_on_a*a->invM; Vector3d a_angular = MVMult(a->I,torque_on_a).CrossProduct(ra);
	Vector3d b_linear = force_on_b*b->invM; Vector3d b_angular = MVMult(b->I,torque_on_b).CrossProduct(rb);

	return ni.DotProduct((a_linear+a_angular)-(b_linear+b_angular));
}

void compute_a_matrix(vector<contact*> contacts, int contactnbr, float **a){
	// compute matrix A used to calculate forces on resting contacts
	int i,j;
	for (i = 0; i < contactnbr; i++){
		for (j = 0; j < contactnbr; j++){
			(*a)[i*contactnbr+j] = compute_aij(contacts[i],contacts[j]);
		}
	}
}

bool separates(Vector3d p, Vector3d n, RIGID_BODY *r1, RIGID_BODY *r2){
	
	// check whether the plane defined by point p and normal n separates
	// the two rigid bodies r1 and r2.

	int i;

	if (n.DotProduct(r1->xcm-p)*n.DotProduct(r2->xcm-p)>=0.0){
		return false;
	}

	for (i = 0; i < r1->nbr_of_verts; i++){
		if (n.DotProduct(r1->vertices[i] - p) <= 0.0){
			return false;
		}
	}

	return true;
}

bool triangle_triangle_intersection(int face_i, int face_j, RIGID_BODY *a, RIGID_BODY *b){

	// check whether two triangles defined by faces i and j of rigid bodies a and b may intersect
	// return false if all points of one triangle are on the same side of the plane defined
	// by the other triangle

	Vector3d p1,p2,p3; Vector3d q1,q2,q3; face face_a, face_b; Vector3d d1,d2,n;
	face_a = a->faces[face_i]; face_b = b->faces[face_j];

	p1 = a->vertices[face_a.v1]; p2 = a->vertices[face_a.v2]; p3 = a->vertices[face_a.v3];
	q1 = b->vertices[face_b.v1]; q2 = b->vertices[face_b.v2]; q3 = b->vertices[face_b.v3];

	d1 = p2-p1; d2 = p3-p1;
	n = d1.CrossProduct(d2);

	if ((n.DotProduct(q1-p1) < 0)&&(n.DotProduct(q2-p1) < 0)&&(n.DotProduct(q3-p1) < 0)){
		return false;
	}else if ((n.DotProduct(q1-p1) > 0)&&(n.DotProduct(q2-p1) > 0)&&(n.DotProduct(q3-p1) > 0)){
		return false;
	}

	d1 = q2-q1; d2 = q3-q1;
	n = d1.CrossProduct(d2);

	if ((n.DotProduct(p1-q1) < 0)&&(n.DotProduct(p2-q1) < 0)&&(n.DotProduct(p3-q1) < 0)){
		return false;
	}else if ((n.DotProduct(p1-q1) > 0)&&(n.DotProduct(p2-q1) > 0)&&(n.DotProduct(p3-q1) > 0)){
		return false;
	}

	return true;
}

bool vertex_face_collision(Vector3d p1, Vector3d p2, Vector3d p3, Vector3d vertex, Vector3d acenter, contact *cont){
	
	// check for vertex-face-collision by testing whether the line from 'acenter' (center of rigid body a) to 'vertex'
	// intersects the triangle given by points p1,p2 and p3 and normal n. store results in cont

	Vector3d d1 = p3-p2; 
	Vector3d d2 = p1-p2; 
	Vector3d n = d1.CrossProduct(d2);
	n.normalize();

	if (n.DotProduct(vertex-p1)*n.DotProduct(acenter-p1)>0.0){
		return false;
	}

	Vector3d d3; Vector3d hit; Vector3d c;
	Vector3d d; 
	float t = 0.0;
	float s1, s2, s3;
	d = acenter - vertex;

	if (d.DotProduct(n)!= 0){
		t = (p1.DotProduct(n)-vertex.DotProduct(n))/d.DotProduct(n);

		if ((t >= 0.0)&&(t <= 1.0)){

			hit = vertex + d*t; 

			d1 = p1 - hit;
			d2 = p2 - hit;
			d3 = p3 - hit;

			s1 = (d2.CrossProduct(d3)).DotProduct(n)/2.0;
			s2 = (d3.CrossProduct(d1)).DotProduct(n)/2.0;
			s3 = (d1.CrossProduct(d2)).DotProduct(n)/2.0;

			if ((s1>=0.0)&&(s2>=0.0)&&(s3>=0.0)){ 
				if (n.DotProduct(d) < 0.0){
					n = -n;
				}
				cont->n = n; cont->vf = true; cont->vertex = vertex;
				cont->p = hit;
				return true;
			}
		}
	}

	if (n.DotProduct(d) < 0.0){
		n = -n;
	}
	return false;
	
}

bool edge_edge_collision(Vector3d epa1, Vector3d epa2, Vector3d centera, Vector3d epb1, Vector3d epb2, contact *cont){
	
	// check "whether two edges collide", given by points epa1, epa2 and points epb1, epb2.
	// test if the line given by epb2 and epb1 intersects the triangle given by centera (center of a rigid body)
	// and epa1, epa2. store results in cont.
	
	Vector3d n; Vector3d d1; Vector3d d2; Vector3d d3; Vector3d hit; Vector3d c;
	Vector3d d; 
	float t = 0.0;
	float s1, s2, s3;
	d = epb2 - epb1;
	d1 = centera - epa2; 
	d2 = epa1 - epa2; 
	n = d1.CrossProduct(d2);
	n.normalize();
	cont->ea = epa2-epa1; cont->eb = epb2-epb1; cont->vf = false;
	cont->n = cont->ea.CrossProduct(cont->eb); cont->n.normalize();
	if (d.DotProduct(n)!= 0){
		t = (epa1.DotProduct(n)-epb1.DotProduct(n))/d.DotProduct(n);

		if ((t >= 0.0)&&(t <= 1.0)){

			hit = epb1 + d*t; 

			d1 = epa1 - hit;
			d2 = epa2 - hit;
			d3 = centera - hit;

			s1 = (d2.CrossProduct(d3)).DotProduct(n)/2.0;
			s2 = (d3.CrossProduct(d1)).DotProduct(n)/2.0;
			s3 = (d1.CrossProduct(d2)).DotProduct(n)/2.0;

			if ((s1>=0.0)&&(s2>=0.0)&&(s3>=0.0)){
				cont->p = hit;
				return true;
			}
		}
	}
	return false;
}

bool find_separating_plane(RIGID_BODY *a, RIGID_BODY *b, contact *cont){

	// search for a plane, defined by one of the faces, that separates a and b.

	int i; Vector3d normal;
	Vector3d v1,v2,v3;

	for (i = 0; i < b->nbr_of_faces; i++){		
		v1 = b->vertices[b->faces[i].v1]; v2 = b->vertices[b->faces[i].v2]; v3 = b->vertices[b->faces[i].v3];
		normal = (v2-v1).CrossProduct(v3-v1); normal.normalize();
		if (normal.DotProduct(b->radii[b->faces[i].v1])<0.0){
			normal = -normal;
		}
		if (separates(v1,normal,a,b)){
			cont->facenbr = i; cont->n = normal; cont->vf = true;
			return true;
		}
	}

	return false;
}

bool existing_plane_ok(contact *cont){

	// test if existing plane separates the two rigids bodies in cont. the plane is defined by one of the
	// faces of one of the two rigid bodies.

	if (cont->vf){
		if (separates(cont->b->vertices[cont->b->faces[cont->facenbr].v1],cont->n,cont->a,cont->b)){
			return true;
		}else{
			return false;
		}
	}else{
		return false;
	}
}

void resolve_resting_contacts(){

	int i,j;
	float *ma = new float[resting_contacts_count*resting_contacts_count];
	float *bvec = new float[resting_contacts_count];

	compute_a_matrix(resting_contacts,resting_contacts_count,&ma);
	compute_b_vector(resting_contacts,resting_contacts_count,&bvec);

	_lprec *lp;
	lp = make_lp(resting_contacts_count,resting_contacts_count);

	double *row = new double[resting_contacts_count+1];
	for (i = 0; i < resting_contacts_count; i++){
		for (j = 0; j < resting_contacts_count; j++){
			row[j+1] = ma[i*resting_contacts_count+j];
		}
		add_constraint(lp, row, EQ, bvec[i]);
	}

	for (i = 0; i < resting_contacts_count; i++){
		row[i+1] = 0.0;
	}
	for (i = 0; i < resting_contacts_count; i++){
		row[i+1] = 1.0;
		add_constraint(lp, row, GE, 0.0);
		row[i+1] = 0.0;
	}

	double *forces = new double[resting_contacts_count];

	for (i = 0; i < resting_contacts_count; i++){
			row[i+1] = 1.0;
		}
	set_obj_fn(lp, row);

	solve(lp);
	get_variables(lp, forces);

	//if (solve(lp) == 0){
	//	get_variables(lp, forces);
	//}else{
	//	delete_lp(lp);
	//	lp = make_lp(resting_contacts_count,resting_contacts_count);
	//	for (i = 0; i < resting_contacts_count; i++){
	//		for (j = 0; j < resting_contacts_count; j++){
	//			row[j+1] = ma[i*resting_contacts_count+j];
	//		}
	//		add_constraint(lp, row, GE, bvec[i]);
	//	}

	//	for (i = 0; i < resting_contacts_count; i++){
	//		row[i+1] = 0.0;
	//	}
	//	for (i = 0; i < resting_contacts_count; i++){
	//		row[i+1] = 1.0;
	//		add_constraint(lp, row, GE, 0.0);
	//		row[i+1] = 0.0;
	//	}
	//	for (i = 0; i < resting_contacts_count; i++){
	//		row[i+1] = 1.0;
	//	}
	//	set_obj_fn(lp, row);
	//	solve(lp);
	//	get_variables(lp, forces);
	//	delete_lp(lp);
	//	lp = make_lp(resting_contacts_count,resting_contacts_count);
	//	float temp; double *row2 = new double[resting_contacts_count+1]; double *row3  = new double[resting_contacts_count+1];
	//	double *row4  = new double[resting_contacts_count+1];
	//	for (i = 0; i < resting_contacts_count; i++){
	//		temp = 0.0;
	//		for (j = 0; j < resting_contacts_count; j++){
	//			temp += ma[i*resting_contacts_count+j]*forces[j];
	//		}
	//		temp -= bvec[i];
	//		if (temp > 0.0000001){
	//			// vanishing
	//			row[i+1] = 0.0;
	//			row2[i+1] = 1.0;
	//		}else{
	//			// not vanishing
	//			row[i+1] = 1.0;
	//			row2[i+1] = 0.0;
	//		}
	//	}
	//	add_constraint(lp, row, GE, 0.0);
	//	add_constraint(lp, row2, EQ, 0.0);
	//	for (i = 0; i < resting_contacts_count; i++){
	//		for (j = 0; j < resting_contacts_count; j++){
	//			row3[j+1] = ma[i*resting_contacts_count+j];
	//		}
	//		if (row2[i+1] == 1.0){
	//			// vanishing
	//			add_constraint(lp, row3, GE, bvec[i]);
	//		}else{
	//			add_constraint(lp, row3, EQ, bvec[i]);
	//		}
	//	}
	//	solve(lp);
	//	get_variables(lp, forces);
	//}

	Vector3d n; RIGID_BODY *a, *b;
	for (i = 0; i < resting_contacts_count; i++){
		n = resting_contacts[i]->n;
		a = resting_contacts[i]->a; b = resting_contacts[i]->b;
		a->F = a->F + n*forces[i];
		a->torque = a->torque + (resting_contacts[i]->p - a->xcm).CrossProduct(n*forces[i]);
		b->F = b->F - n*forces[i];
		b->torque = b->torque - (resting_contacts[i]->p - b->xcm).CrossProduct(n*forces[i]);
	}

    delete_lp(lp);
	delete[] ma; delete[] bvec; delete []forces; delete[] row;

	resting_contacts.clear();
	resting_contacts_count = 0;
}

void resolve_collision_from_contact(contact *cont, Vector3d vcma, Vector3d vcmb){

	// resolve collision by using information given in cont.

	RIGID_BODY *rba = cont->a;
	RIGID_BODY *rbb = cont->b;
	Vector3d pa; Vector3d pb; Vector3d ra; Vector3d rb;
	
	ra = cont->p - rba->xcm; rb = cont->p - rbb->xcm;
	pa = vcma + rba->w.CrossProduct(ra);
	pb = vcmb + rbb->w.CrossProduct(rb);

	float e = 0.726;
	float vrel = cont->n.DotProduct(pa-pb);

	if (vrel > 0.0){
		return;
	}

	if (vrel > -0.2){
	//	contact *temp = new contact(cont);
	//	resting_contacts.push_back(temp);
	//	resting_contacts_count++;
		return;
	}

	if (colliding_contact_solved){
		return; 
	}

	float denom = rba->invM + rbb->invM + ((MVMult(rba->I,ra.CrossProduct(cont->n))).CrossProduct(ra)
		+ (MVMult(rbb->I,rb.CrossProduct(cont->n))).CrossProduct(rb)).DotProduct(cont->n);
	float j = (-(1+e)*vrel)/denom;
	Vector3d J = cont->n*j;
	rba->L = rba->L + ra.CrossProduct(J);
	rba->vcm = rba->vcm + J*rba->invM;
	rba->w.set(Vector3d(rba->I[0]).DotProduct(rba->L),Vector3d(rba->I[1]).DotProduct(rba->L),Vector3d(rba->I[2]).DotProduct(rba->L));
	J = -J;
	rbb->L = rbb->L + rb.CrossProduct(J);
	rbb->vcm = rbb->vcm + J*rbb->invM;
	rbb->w.set(Vector3d(rbb->I[0]).DotProduct(rbb->L),Vector3d(rbb->I[1]).DotProduct(rbb->L),Vector3d(rbb->I[2]).DotProduct(rbb->L));
	
	colliding_contact_solved = true;
}

void resolve_collision_from_scratch(RIGID_BODY *a, RIGID_BODY *b, contact *cont){

	// check where the collision occurs. this information will then be stored in
	// cont and used by resolve_collision_from_contact.

	int i, j; Vector3d normal; Vector3d no1; Vector3d no2; Vector3d no3; 
	Vector3d vcma = Vector3d(a->vcm); Vector3d vcmb = Vector3d(b->vcm);
	colliding_contact_solved = false;

	for (i = 0; i < b->nbr_of_faces; i++){
		for (j = 0; j < a->nbr_of_verts; j++){	
			if (vertex_face_collision(b->vertices[b->faces[i].v1],b->vertices[b->faces[i].v2],b->vertices[b->faces[i].v3], a->vertices[j], a->xcm,cont)){
				cont->facenbr = i; cont->vert = j;
				resolve_collision_from_contact(cont,vcma,vcmb);
			}
		}
	}

	RIGID_BODY *temp = cont->a; cont->a = cont->b; cont->b = temp;
	for (i = 0; i < a->nbr_of_faces; i++){
		for (j = 0; j < b->nbr_of_verts; j++){	
			if (vertex_face_collision(a->vertices[a->faces[i].v1],a->vertices[a->faces[i].v2],a->vertices[a->faces[i].v3], b->vertices[j], b->xcm,cont)){
				cont->facenbr = i; cont->vert = j;
				resolve_collision_from_contact(cont,vcmb,vcma);
			}
		}
	}
	temp = cont->a; cont->a = cont->b; cont->b = temp;

	Vector3d centera = a->xcm; Vector3d centerb = b->xcm; Vector3d epa1; Vector3d epa2; Vector3d epb1; Vector3d epb2;

	for (i = 0; i < a->nbr_of_faces; i++){
		for (j = 0; j < b->nbr_of_faces; j++){

			if (triangle_triangle_intersection(i,j,a,b)){
				epa1 = a->vertices[a->faces[i].v1]; epa2 = a->vertices[a->faces[i].v2];
				epb1 = b->vertices[b->faces[j].v1];	epb2 = b->vertices[b->faces[j].v2];
				if(edge_edge_collision(epb1,epb2,centerb,epa1,epa2,cont)){
					if(edge_edge_collision(epa1,epa2,centera,epb1,epb2,cont)){
						resolve_collision_from_contact(cont,vcma,vcmb);
					}
				}
				epb1 = b->vertices[b->faces[j].v1];	epb2 = b->vertices[b->faces[j].v3];
				if(edge_edge_collision(epb1,epb2,centerb,epa1,epa2,cont)){
					if(edge_edge_collision(epa1,epa2,centera,epb1,epb2,cont)){
						resolve_collision_from_contact(cont,vcma,vcmb);
					}
				}
				epb1 = b->vertices[b->faces[j].v2];	epb2 = b->vertices[b->faces[j].v3];
				if(edge_edge_collision(epb1,epb2,centerb,epa1,epa2,cont)){
					if(edge_edge_collision(epa1,epa2,centera,epb1,epb2,cont)){
						resolve_collision_from_contact(cont,vcma,vcmb);
					}
				}	

				epa1 = a->vertices[a->faces[i].v1];	epa2 = a->vertices[a->faces[i].v3];
				epb1 = b->vertices[b->faces[j].v1];	epb2 = b->vertices[b->faces[j].v2];
				if(edge_edge_collision(epb1,epb2,centerb,epa1,epa2,cont)){
					if(edge_edge_collision(epa1,epa2,centera,epb1,epb2,cont)){
						resolve_collision_from_contact(cont,vcma,vcmb);
					}
				}
				epb1 = b->vertices[b->faces[j].v1];	epb2 = b->vertices[b->faces[j].v3];
				if(edge_edge_collision(epb1,epb2,centerb,epa1,epa2,cont)){
					if(edge_edge_collision(epa1,epa2,centera,epb1,epb2,cont)){
						resolve_collision_from_contact(cont,vcma,vcmb);
					}
				}
				epb1 = b->vertices[b->faces[j].v2];	epb2 = b->vertices[b->faces[j].v3];
				if(edge_edge_collision(epb1,epb2,centerb,epa1,epa2,cont)){
					if(edge_edge_collision(epa1,epa2,centera,epb1,epb2,cont)){
						resolve_collision_from_contact(cont,vcma,vcmb);
					}
				}	

				epa1 = a->vertices[a->faces[i].v2];	epa2 = a->vertices[a->faces[i].v3];
				epb1 = b->vertices[b->faces[j].v1];	epb2 = b->vertices[b->faces[j].v2];
				if(edge_edge_collision(epb1,epb2,centerb,epa1,epa2,cont)){
					if(edge_edge_collision(epa1,epa2,centera,epb1,epb2,cont)){
						resolve_collision_from_contact(cont,vcma,vcmb);
					}
				}
				epb1 = b->vertices[b->faces[j].v1];	epb2 = b->vertices[b->faces[j].v3];
				if(edge_edge_collision(epb1,epb2,centerb,epa1,epa2,cont)){
					if(edge_edge_collision(epa1,epa2,centera,epb1,epb2,cont)){
						resolve_collision_from_contact(cont,vcma,vcmb);
					}
				}
				epb1 = b->vertices[b->faces[j].v2];	epb2 = b->vertices[b->faces[j].v3];
				if(edge_edge_collision(epb1,epb2,centerb,epa1,epa2,cont)){
					if(edge_edge_collision(epa1,epa2,centera,epb1,epb2,cont)){
						resolve_collision_from_contact(cont,vcma,vcmb);
					}
				}
			}
		}
	}
}

bool check_for_collisions_and_update(){

	// check all pairs of rigid bodies for collisions. if their bounding boxes overlap
	// further tests are needed.


	int i,j,k; 
	k = 0;
	for (i = 0; i < nbr_of_rigids; i++){
		for (j = i+1; j < nbr_of_rigids; j++){
			if (rigids[i]->box->overlap(rigids[j]->box)){
				contact *sep = separators[k];
				if (!existing_plane_ok(sep)){
					if (!find_separating_plane(rigids[i],rigids[j],sep)){
						resolve_collision_from_scratch(rigids[i],rigids[j],sep);
					}
				}
			}
			k++;
		}
	}
	return true;
}

void create_collision_object(char *filename, int id, float M){

	// create the object that will be used for resolving collisions (simpler representation may be used here)

	ifstream input_file; 
	int nbr_of_verts; int nbr_of_normals; int nbr_of_faces; 
	int i; char temp = 0x00;
  
	input_file.open(filename);
	if(!input_file) 
	{
		cerr << " opening failed!" << endl;
		return;
	}

	for (i = 0; temp != 'r'; i++){
		input_file >> temp;
	}
	input_file >> nbr_of_verts;
	float *verts = new float[nbr_of_verts*3];
	for (i = 0; i < nbr_of_verts; i++){
		input_file >> temp;
		input_file >> verts[3*i];
		input_file >> verts[3*i+1];
		input_file >> verts[3*i+2];
	 }
	for (i = 0; temp != 'r'; i++){
		input_file >> temp;
	 }
	input_file >> nbr_of_normals;
	float *norms = new float[nbr_of_normals*3];
	for (i = 0; i < nbr_of_normals; i++){
		input_file >> temp; input_file >> temp;
		input_file >> norms[3*i];
		input_file >> norms[3*i+1];
		input_file >> norms[3*i+2];
	}
	for (i = 0; temp != 'r'; i++){
		input_file >> temp;
	}
	input_file >> nbr_of_faces;
	face *faces = new face[nbr_of_faces];
	for (i = 0; i < nbr_of_faces; i++){
		input_file >> temp;
		input_file >> faces[i].v1; input_file >> temp; input_file >> temp; input_file >> faces[i].n1;
		input_file >> faces[i].v2; input_file >> temp; input_file >> temp; input_file >> faces[i].n2;
		input_file >> faces[i].v3; input_file >> temp; input_file >> temp; input_file >> faces[i].n3;
		faces[i].v1 -= 1; faces[i].n1 -= 1;
		faces[i].v2 -= 1; faces[i].n2 -= 1;
		faces[i].v3 -= 1; faces[i].n3 -= 1;
	}
  
	input_file.close();

	GLfloat A[3][3] = {1.0,0.0,0.0,0.0,1.0,0.0,0.0,0.0,1.0};
	Vector3d L = Vector3d(0.0,0.0,0.0);
	Vector3d v0 = Vector3d(0.0,0.0,0.0);
	Vector3d *vertices;
	vertices = new Vector3d[nbr_of_verts];
	for (i = 0; i < nbr_of_verts; i++){
		vertices[i] = Vector3d(verts[3*i],verts[3*i+1],verts[3*i+2]);
	}
	Vector3d *normals;
	normals = new Vector3d[nbr_of_normals];
	for (i = 0; i < nbr_of_normals; i++){
		normals[i] = Vector3d(norms[3*i],norms[3*i+1],norms[3*i+2]);
	}

	OBB *obb = new OBB();
	RIGID_BODY *rb = new RIGID_BODY(vertices,nbr_of_verts,faces,nbr_of_faces,normals,nbr_of_normals,M,v0,A,L,obb,id);
	rigids.push_back(rb);
	nbr_of_rigids++;
}

void create_object_representation(char *filename, int id){

	// create object that will represent the rigid body and will be rendered.

	ifstream input_file; 
	int nbr_of_verts; int nbr_of_normals; int nbr_of_faces; 
	int i; char temp = 0x00;
  
	input_file.open(filename);
	if(!input_file) 
	{
		cerr << " opening failed!" << endl;
		return;
	}

	for (i = 0; temp != 'r'; i++){
		input_file >> temp;
	}
	input_file >> nbr_of_verts;
	float *verts = new float[nbr_of_verts*3];
	for (i = 0; i < nbr_of_verts; i++){
		input_file >> temp;
		input_file >> verts[3*i];
		input_file >> verts[3*i+1];
		input_file >> verts[3*i+2];
	 }
	for (i = 0; temp != 'r'; i++){
		input_file >> temp;
	 }
	input_file >> nbr_of_normals;
	float *norms = new float[nbr_of_normals*3];
	for (i = 0; i < nbr_of_normals; i++){
		input_file >> temp; input_file >> temp;
		input_file >> norms[3*i];
		input_file >> norms[3*i+1];
		input_file >> norms[3*i+2];
	}
	for (i = 0; temp != 'r'; i++){
		input_file >> temp;
	}
	input_file >> nbr_of_faces;
	face *faces = new face[nbr_of_faces];
	for (i = 0; i < nbr_of_faces; i++){
		input_file >> temp;
		input_file >> faces[i].v1; input_file >> temp; input_file >> temp; input_file >> faces[i].n1;
		input_file >> faces[i].v2; input_file >> temp; input_file >> temp; input_file >> faces[i].n2;
		input_file >> faces[i].v3; input_file >> temp; input_file >> temp; input_file >> faces[i].n3;
		faces[i].v1 -= 1; faces[i].n1 -= 1;
		faces[i].v2 -= 1; faces[i].n2 -= 1;
		faces[i].v3 -= 1; faces[i].n3 -= 1;
	}
  
	input_file.close();

	glVertexPointer(3,GL_FLOAT,0,verts);
	glNormalPointer(GL_FLOAT,0,norms);
	glNewList(id, GL_COMPILE);
	for (i = 0; i < nbr_of_faces; i++){
		glBegin(GL_TRIANGLES);
		glNormal3fv(norms + 3*(faces[i].n1));
		glVertex3fv(verts + 3*(faces[i].v1));
		glNormal3fv(norms + 3*(faces[i].n2));
		glVertex3fv(verts + 3*(faces[i].v2));
		glNormal3fv(norms + 3*(faces[i].n3));
		glVertex3fv(verts + 3*(faces[i].v3));
		glEnd();
	}
	glEndList();
}

void load_and_create_object(char *filename, char *filename2, int id, float M){

	// load and create the object by using the object defined in 'filename' to resolve collisions
	// and the object defined in 'filename2' to render it.

	create_object_representation(filename2,id);
	create_collision_object(filename,id,M);
}

void put_cube_at(Vector3d pos){

	load_and_create_object("cube.obj", "cube.obj", nbr_of_rigids+1, 1.0);
	float size[3] = {0.5,0.5,0.5};
	float w[3] = {1.0,0.0,0.0};
	float h[3] = {0.0,1.0,0.0};
	float d[3] = {0.0,0.0,1.0};
	float center[3] = {0.5,0.5,0.5};
	rigids[nbr_of_rigids-1]->box->set(size,w,h,d,center);
	rigids[nbr_of_rigids-1]->xcm = rigids[nbr_of_rigids-1]->xcm + pos;
	rigids[nbr_of_rigids-1]->box->translate(rigids[nbr_of_rigids-1]->xcm);
}

void put_sphere_at(Vector3d pos){

	load_and_create_object("sphere.obj", "sphere.obj", nbr_of_rigids+1, 1.0);
	float size4[3] = {1.1,1.1,1.1};
	float w4[3] = {1.0,0.0,0.0};
	float ah4[3] = {0.0,1.0,0.0};
	float d4[3] = {0.0,0.0,1.0};
	float center4[3] = {0.0,0.0,0.0};
	rigids[nbr_of_rigids-1]->box->set(size4,w4,ah4,d4,center4);
	rigids[nbr_of_rigids-1]->xcm = rigids[nbr_of_rigids-1]->xcm + pos;
	rigids[nbr_of_rigids-1]->box->translate(rigids[nbr_of_rigids-1]->xcm);
}

void put_cylinder_at(Vector3d pos){

	load_and_create_object("cylinder.obj", "cylinder.obj", nbr_of_rigids+1, 1.0);
	float size2[3] = {1.1,1.1,1.1};
	float w2[3] = {1.0,0.0,0.0};
	float ah2[3] = {0.0,1.0,0.0};
	float d2[3] = {0.0,0.0,1.0};
	float center2[3] = {0.0,0.0,0.0};
	rigids[nbr_of_rigids-1]->box->set(size2,w2,ah2,d2,center2);
	rigids[nbr_of_rigids-1]->xcm = rigids[nbr_of_rigids-1]->xcm + pos;
	rigids[nbr_of_rigids-1]->box->translate(rigids[nbr_of_rigids-1]->xcm);

}

void put_cone_at(Vector3d pos){

	load_and_create_object("cone.obj", "cone.obj", nbr_of_rigids+1, 1.0);
	float size2[3] = {1.1,1.1,1.1};
	float w2[3] = {1.0,0.0,0.0};
	float ah2[3] = {0.0,1.0,0.0};
	float d2[3] = {0.0,0.0,1.0};
	float center2[3] = {0.0,0.0,0.0};
	rigids[nbr_of_rigids-1]->box->set(size2,w2,ah2,d2,center2);
	rigids[nbr_of_rigids-1]->xcm = rigids[nbr_of_rigids-1]->xcm + pos;
	rigids[nbr_of_rigids-1]->box->translate(rigids[nbr_of_rigids-1]->xcm);

}

void put_monkey_at(Vector3d pos){

	load_and_create_object("monkey.obj", "monkey.obj", nbr_of_rigids+1, 1.0);
	float size2[3] = {1.1,1.1,1.1};
	float w2[3] = {1.0,0.0,0.0};
	float ah2[3] = {0.0,1.0,0.0};
	float d2[3] = {0.0,0.0,1.0};
	float center2[3] = {0.0,0.0,0.0};
	rigids[nbr_of_rigids-1]->box->set(size2,w2,ah2,d2,center2);
	rigids[nbr_of_rigids-1]->xcm = rigids[nbr_of_rigids-1]->xcm + pos;
	rigids[nbr_of_rigids-1]->box->translate(rigids[nbr_of_rigids-1]->xcm);

}


void init(void){
	
	GLfloat light_position[] = {1.0,2.0,1.0,0.0};
	GLfloat white_light[] = {1.0,1.0,1.0,1.0};
	GLfloat lmodel_ambient[] = {1.0,1.0,1.0,1.0};
	glClearColor(0.0,0.0,0.0,0.0);
	glShadeModel(GL_SMOOTH);
	glLightfv(GL_LIGHT0, GL_POSITION, light_position);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, white_light);
	glLightfv(GL_LIGHT0, GL_SPECULAR, white_light);
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, lmodel_ambient);
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_COLOR_MATERIAL_FACE);
	glEnable(GL_NORMALIZE);
	glEnableClientState(GL_VERTEX_ARRAY); 

	put_cube_at(Vector3d(0.5,1.0,0.2));
	put_cube_at(Vector3d(2.5,3.0,-0.2));
	put_cube_at(Vector3d(-1.5,2.0,0.0));
	put_cube_at(Vector3d(-2.5,4.5,0.0));
	put_cube_at(Vector3d(-1.5,6.0,-1.5));
	put_cube_at(Vector3d(1.0,1.7,1.4));
	put_cube_at(Vector3d(4.5,3.5,0.0));
	put_cube_at(Vector3d(3.5,5.5,1.0));
	put_cube_at(Vector3d(0.8,5.0,-1.0));
	put_cube_at(Vector3d(-3.5,1.5,0.5));
	put_cube_at(Vector3d(1.5,6.5,0.3));
	put_cube_at(Vector3d(2.5,6.0,-0.5));
	put_cube_at(Vector3d(2.7,9.5,0.2));
	put_cube_at(Vector3d(-2.6,9.1,0.7));
	put_cube_at(Vector3d(0.2,8.5,-2.5));

	put_cube_at(Vector3d(0.5,11.0,0.2));
	put_cube_at(Vector3d(2.5,13.0,-0.2));
	put_cube_at(Vector3d(-1.5,12.0,0.0));
	put_cube_at(Vector3d(-2.5,14.5,0.0));
	put_cube_at(Vector3d(-1.5,16.0,-1.5));
	put_cube_at(Vector3d(1.0,11.7,1.4));
	put_cube_at(Vector3d(4.5,13.5,0.0));
	put_cube_at(Vector3d(3.5,15.5,1.0));
	put_cube_at(Vector3d(0.8,15.0,-1.0));
	put_cube_at(Vector3d(-3.5,11.5,0.5));
	put_cube_at(Vector3d(1.5,16.5,0.3));
	put_cube_at(Vector3d(2.5,16.0,-0.5));
	put_cube_at(Vector3d(2.7,19.5,0.2));
	put_cube_at(Vector3d(-2.6,19.1,0.7));
	put_cube_at(Vector3d(0.2,18.5,-2.5));

	put_cube_at(Vector3d(0.5,21.0,0.2));
	put_cube_at(Vector3d(2.5,23.0,-0.2));
	put_cube_at(Vector3d(-1.5,22.0,0.0));
	put_cube_at(Vector3d(-2.5,24.5,0.0));
	put_cube_at(Vector3d(-1.5,26.0,-1.5));
	put_cube_at(Vector3d(1.0,21.7,1.4));
	put_cube_at(Vector3d(4.5,23.5,0.0));
	put_cube_at(Vector3d(3.5,25.5,1.0));
	put_cube_at(Vector3d(0.8,25.0,-1.0));
	put_cube_at(Vector3d(-3.5,21.5,0.5));
	put_cube_at(Vector3d(1.5,26.5,0.3));
	put_cube_at(Vector3d(2.5,26.0,-0.5));
	put_cube_at(Vector3d(2.7,29.5,0.2));
	put_cube_at(Vector3d(-2.6,29.1,0.7));
	put_cube_at(Vector3d(0.2,28.5,-2.5));

	put_cube_at(Vector3d(0.5,31.0,0.2));
	put_cube_at(Vector3d(2.5,33.0,-0.2));
	put_cube_at(Vector3d(-1.5,32.0,0.0));
	put_cube_at(Vector3d(-2.5,34.5,0.0));
	put_cube_at(Vector3d(-1.5,36.0,-1.5));
	put_cube_at(Vector3d(1.0,31.7,1.4));
	put_cube_at(Vector3d(4.5,33.5,0.0));
	put_cube_at(Vector3d(3.5,35.5,1.0));
	put_cube_at(Vector3d(0.8,35.0,-1.0));
	put_cube_at(Vector3d(-3.5,31.5,0.5));
	put_cube_at(Vector3d(1.5,36.5,0.3));
	put_cube_at(Vector3d(2.5,36.0,-0.5));
	put_cube_at(Vector3d(2.7,39.5,0.2));
	put_cube_at(Vector3d(-2.6,39.1,0.7));
	put_cube_at(Vector3d(0.2,38.5,-2.5));

	
	put_cube_at(Vector3d(0.5,1.0,20.2));
	rigids[nbr_of_rigids-1]->vcm = Vector3d(0.0,0.0,-25.0);
	put_cube_at(Vector3d(2.5,3.0,19.8));
	rigids[nbr_of_rigids-1]->vcm = Vector3d(0.0,0.0,-25.0);
	put_cube_at(Vector3d(-1.5,2.0,20.0));
	rigids[nbr_of_rigids-1]->vcm = Vector3d(0.0,0.0,-25.0);
	put_cube_at(Vector3d(-2.5,4.5,20.0));
	rigids[nbr_of_rigids-1]->vcm = Vector3d(0.0,0.0,-25.0);
	put_cube_at(Vector3d(-1.5,6.0,18.5));
	rigids[nbr_of_rigids-1]->vcm = Vector3d(0.0,0.0,-25.0);
	put_cube_at(Vector3d(1.0,1.7,21.4));
	rigids[nbr_of_rigids-1]->vcm = Vector3d(0.0,0.0,-25.0);
	put_cube_at(Vector3d(4.5,3.5,20.0));
	rigids[nbr_of_rigids-1]->vcm = Vector3d(0.0,0.0,-25.0);
	put_cube_at(Vector3d(3.5,5.5,21.0));
	rigids[nbr_of_rigids-1]->vcm = Vector3d(0.0,0.0,-25.0);
	put_cube_at(Vector3d(0.8,5.0,19.0));
	rigids[nbr_of_rigids-1]->vcm = Vector3d(0.0,0.0,-25.0);
	put_cube_at(Vector3d(-3.5,1.5,20.5));
	rigids[nbr_of_rigids-1]->vcm = Vector3d(0.0,0.0,-25.0);
	put_cube_at(Vector3d(1.5,6.5,20.3));
	rigids[nbr_of_rigids-1]->vcm = Vector3d(0.0,0.0,-25.0);
	put_cube_at(Vector3d(2.5,6.0,19.5));
	rigids[nbr_of_rigids-1]->vcm = Vector3d(0.0,0.0,-25.0);
	put_cube_at(Vector3d(2.7,9.5,20.2));
	rigids[nbr_of_rigids-1]->vcm = Vector3d(0.0,0.0,-25.0);
	put_cube_at(Vector3d(-2.6,9.1,20.7));
	rigids[nbr_of_rigids-1]->vcm = Vector3d(0.0,0.0,-25.0);
	put_cube_at(Vector3d(0.2,8.5,17.5));
	rigids[nbr_of_rigids-1]->vcm = Vector3d(0.0,0.0,-25.0);

	put_cube_at(Vector3d(0.5,6.0,20.2));
	rigids[nbr_of_rigids-1]->vcm = Vector3d(0.0,0.0,-25.0);
	put_cube_at(Vector3d(2.5,8.0,19.8));
	rigids[nbr_of_rigids-1]->vcm = Vector3d(0.0,0.0,-25.0);
	put_cube_at(Vector3d(-1.5,7.0,20.0));
	rigids[nbr_of_rigids-1]->vcm = Vector3d(0.0,0.0,-25.0);
	put_cube_at(Vector3d(-2.5,9.5,20.0));
	rigids[nbr_of_rigids-1]->vcm = Vector3d(0.0,0.0,-25.0);
	put_cube_at(Vector3d(-1.5,11.0,18.5));
	rigids[nbr_of_rigids-1]->vcm = Vector3d(0.0,0.0,-25.0);
	put_cube_at(Vector3d(1.0,6.7,21.4));
	rigids[nbr_of_rigids-1]->vcm = Vector3d(0.0,0.0,-25.0);
	put_cube_at(Vector3d(4.5,8.5,20.0));
	rigids[nbr_of_rigids-1]->vcm = Vector3d(0.0,0.0,-25.0);
	put_cube_at(Vector3d(3.5,10.5,21.0));
	rigids[nbr_of_rigids-1]->vcm = Vector3d(0.0,0.0,-25.0);
	put_cube_at(Vector3d(0.8,10.0,19.0));
	rigids[nbr_of_rigids-1]->vcm = Vector3d(0.0,0.0,-25.0);
	put_cube_at(Vector3d(-3.5,6.5,20.5));
	rigids[nbr_of_rigids-1]->vcm = Vector3d(0.0,0.0,-25.0);
	put_cube_at(Vector3d(1.5,11.5,20.3));
	rigids[nbr_of_rigids-1]->vcm = Vector3d(0.0,0.0,-25.0);
	put_cube_at(Vector3d(2.5,11.0,19.5));
	rigids[nbr_of_rigids-1]->vcm = Vector3d(0.0,0.0,-25.0);
	put_cube_at(Vector3d(2.7,14.5,20.2));
	rigids[nbr_of_rigids-1]->vcm = Vector3d(0.0,0.0,-25.0);
	put_cube_at(Vector3d(-2.6,14.1,20.7));
	rigids[nbr_of_rigids-1]->vcm = Vector3d(0.0,0.0,-25.0);
	put_cube_at(Vector3d(0.2,13.5,17.5));
	rigids[nbr_of_rigids-1]->vcm = Vector3d(0.0,0.0,-25.0);


	//load_and_create_object("plane.obj", "plane.obj", nbr_of_rigids+1, 0.0);
	//float size6[3] = {5.0,0.1,5.0};
	//float w6[3] = {1.0,0.0,0.0};
	//float ah6[3] = {0.0,1.0,0.0};
	//float d6[3] = {0.0,0.0,1.0};
	//float center6[3] = {0.0,0.0,0.0};
	//rigids[nbr_of_rigids-1]->box->set(size6,w6,ah6,d6,center6);
	//rigids[nbr_of_rigids-1]->xcm = rigids[nbr_of_rigids-1]->xcm + Vector3d(0.0,-4.0,0.0);
	//rigids[nbr_of_rigids-1]->box->translate(rigids[nbr_of_rigids-1]->xcm);

	load_and_create_object("plane2.obj", "plane2.obj", nbr_of_rigids+1, 0.0);
	float size6[3] = {5.0,1.5,5.0};
	float w6[3] = {1.0,0.0,0.0};
	float ah6[3] = {0.0,1.0,0.0};
	float d6[3] = {0.0,0.0,1.0};
	float center6[3] = {0.0,0.4,0.0};
	rigids[nbr_of_rigids-1]->box->set(size6,w6,ah6,d6,center6);
	rigids[nbr_of_rigids-1]->xcm = rigids[nbr_of_rigids-1]->xcm + Vector3d(0.0,-4.0,0.0);
	rigids[nbr_of_rigids-1]->box->translate(rigids[nbr_of_rigids-1]->xcm);

	//load_and_create_object("hill.obj", "hill.obj", nbr_of_rigids+1, 0.0);
	//float size6[3] = {7.1,7.1,1.2};
	//float w6[3] = {1.0,0.0,0.0};
	//float ah6[3] = {0.0,1.0,0.0};
	//float d6[3] = {0.0,0.0,1.0};
	//float center6[3] = {0.0,-1.0,0.0};
	//rigids[nbr_of_rigids-1]->box->set(size6,w6,ah6,d6,center6);
	//rigids[nbr_of_rigids-1]->xcm = rigids[nbr_of_rigids-1]->xcm + Vector3d(0.0,-4.0,0.0);
	//rigids[nbr_of_rigids-1]->box->translate(rigids[nbr_of_rigids-1]->xcm);

	//rigids[nbr_of_rigids-1]->A[1][1]=0.0; rigids[nbr_of_rigids-1]->A[2][2]=0.0;
	//rigids[nbr_of_rigids-1]->A[1][2]=1.0; rigids[nbr_of_rigids-1]->A[2][1]=-1.0;
	//rigids[nbr_of_rigids-1]->box->transform(rigids[nbr_of_rigids-1]->A);

	for (vector<RIGID_BODY*>::iterator iter = rigids.begin(); iter != rigids.end(); ++iter){
		(*iter)->update_step(0.0);
	}	

	int i,j;
	for (i = 0; i < nbr_of_rigids; i++){
		for (j = i+1; j < nbr_of_rigids; j++){
			contact *temp = new contact(rigids[i],rigids[j]);
			find_separating_plane(rigids[i],rigids[j],temp);
			separators.push_back(temp);
		}
	}	

	force.set(0.0,-20.0,0.0);
}

void redraw(void) {

	float A[3][3];
	float TM[16]; Vector3d c; Vector3d cq;

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	GLfloat color[3] = {1.0,1.0,1.0};
	glMaterialfv(GL_FRONT, GL_DIFFUSE, color);

	for(vector<RIGID_BODY*>::iterator iter = rigids.begin(); iter != rigids.end(); ++iter) {
		(*iter)->update_all_forces(force);
		(*iter)->update_step(dt);
		c = (*iter)->xcm;
		cq = (*iter)->xcmq;
		(*iter)->get_transform(A);
		TM[0] = A[0][0]; TM[4] = A[0][1]; TM[8] = A[0][2]; TM[12] = 0.0;
		TM[1] = A[1][0]; TM[5] = A[1][1]; TM[9] = A[1][2]; TM[13] = 0.0;
		TM[2] = A[2][0]; TM[6] = A[2][1]; TM[10] = A[2][2]; TM[14] = 0.0;
		TM[3] = 0.0; TM[7] = 0.0; TM[11] = 0.0; TM[15] = 1.0;
		glPushMatrix();
		glTranslatef(c[0],c[1],c[2]);
		glMultMatrixf(TM);
		glTranslatef(-cq[0],-cq[1],-cq[2]);
		glCallList((*iter)->id);
		glPopMatrix();
	}

	time += dt;

	check_for_collisions_and_update();
	//if (resting_contacts_count > 0){
	//	resolve_resting_contacts();
	//}
	
	glutSwapBuffers();
	//glFlush();
}

void resize(int w, int h) {

	if(h == 0)
		h = 1;
	GLfloat ratio = 1.0* w / h;
	// Reset the coordinate system before modifying
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	// Set the viewport to be the entire window
	glViewport(0, 0, w, h);
	// Set the correct perspective.
	gluPerspective(45,ratio,1,1000);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	gluLookAt(10.0,0.0,-15.0,
		      0.0,0.0,0.0,
			  0.0f,1.0f,0.0f);
}

void main(int argc, char **argv) {

	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DEPTH | GLUT_DOUBLE | GLUT_RGBA);
	glutInitWindowPosition(100,100);
	glutInitWindowSize(1024,768);
	glutCreateWindow("Rigid_Bodies");
	glutDisplayFunc(redraw);
	glutReshapeFunc(resize);
	glutIdleFunc(redraw);
	init();
	glutMainLoop();
}




